<?php

/*
 * iPay88 Gateway for WooCommerce
 *
 * Description: Allows you to use iPay88 Online Credit Payment with the WooCommerce plugin.
 * Plugin URI: https://www.ipay88.co.id/
 * Author: iPay88 Indonesia
 * License: The MIT License (MIT)
 *
 */

class ipay88_online_credit_Gateway extends ipay88_Abstract_152
{
	//editor: Achmad Baihaqi

	public $id 			= 'ipay88_online_credit';
	public $has_fields 	= true;

	protected $ipay88_payment_method_selected;

	public function __construct()
	{
		parent::__construct();

		$this->icon 				= IPAY88_ONLINE_CREDIT_PLUGIN_URL . '/assets/images/OnlineCredit.png';
		$this->method_title 		= __('iPay88 Online Credit Payment', IPAY88_ONLINE_CREDIT_TEXT_DOMAIN);
		$this->method_description 	= __('Allows you to use iPay88 Online Credit Payment with the WooCommerce plugin.', IPAY88_ONLINE_CREDIT_TEXT_DOMAIN);

		$this->title = isset($this->settings['title']) ? $this->settings['title'] : "iPay88 Online Credit Payment.";
		$this->notify_url = add_query_arg('wc-api', 'ipay88_online_credit_Gateway', home_url('/'));

		// //list on admin multiselect
		// $this->paymenttype_options_ph = array(
		// 	'55'		=> __('Kredivo', IPAY88_ONLINE_CREDIT_TEXT_DOMAIN),
		// 	'70'	=> __('Indodana', IPAY88_ONLINE_CREDIT_TEXT_DOMAIN),
		// 	'71'	=> __('AkuLaku', IPAY88_ONLINE_CREDIT_TEXT_DOMAIN),
		// );

		// Our Actions
		add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
		add_action('woocommerce_receipt_' . $this->id, array($this, 'receipt_page'));
		add_action('woocommerce_api_' . $this->id . '_gateway', array($this, 'ipn_response'));

		$this->hash_amount 		= 0;
		$this->formatted_amount = 0;

		$this->paymenttype_available_ph	= isset($this->settings['paymenttype_available_ph']) ? $this->settings['paymenttype_available_ph'] : array();
	}

	//Initial ipay88 Configs
	function init_configs($isPingback = false)
	{
		ipay88_Config_152::getInstance()->set(array(
			'merchant_code' => $this->settings['merchant_code'],
			'merchant_key' => $this->settings['merchant_key']
		));
	}

	function getConfigOptionByKey($key)
	{
		return $this->settings[$key];
	}

	//@param $order_id
	function receipt_page($order_id)
	{
		$this->init_configs();
		echo $redirect_form = $this->generate_ipay88_form($order_id);
	}

	function dd($data)
	{
		echo "<pre>";
		print_r($data);
		echo "</pre>";
	}

	//Generate iPay88 form
	function generate_ipay88_form($order_id)
	{
		$order = new WC_Order($order_id);
		if ('yes' == $this->debug) WC_iPay88_online_credit::add_debug_log('Generating payment form for order #' . $order_id);

		//Parameters -----------------------------------------------------------------------------------------------

		$merchantCode 	= $this->getConfigOptionByKey('merchant_code');
		$merchantKey 	= $this->getConfigOptionByKey('merchant_key');
		$isTest 		= $this->getConfigOptionByKey('test_mode');

		// $paymentId 		= '55';
		$paymentId 		= $_GET['ipay88_payment_method_selected'];
		$order			= wc_get_order($order_id);
		//$order          = new WC_Order( $order_id );

		$currency 		= get_woocommerce_currency();
		$refNo			= date_format($order->get_date_created(),"Ymd") . "-" .$order_id;
		$isTest 		= $this->getConfigOptionByKey('test_mode');

		if ($isTest == 'no') {
			$getwayUrl = ipay88_Indonesia_152::URL_PAYMENT_PRODUCTION;
		} else {
			$getwayUrl = ipay88_Indonesia_152::URL_PAYMENT_SANDBOX;
		}

		//Format the order total
		$this->format_amount($order->get_total());

		//Amount
		$hashAmount = $this->formatted_amount;

		//Product Desc
		$product_description = '';

		if (0 < sizeof($order->get_items())) {
			foreach ($order->get_items() as $item) {
				if ($item['qty']) {
					$item_name = htmlspecialchars($item['name']);
					// $item_meta = WC_Compat_iPay88_online_credit::get_order_item_meta($item);

					// if ($meta = $item_meta->display(true, true)) {
					// 	$item_name .= ' (' . $meta . ')';
					// }

					$product_description .= $item['qty'] . ' x ' . $item_name . ', ';
				}
			}

			//Add the description
			$product_description = substr($product_description, 0, -2);
		}

		$ProdDesc = $product_description;

		//Customer Data
		$UserName		= $order->billing_first_name . ' ' . $order->billing_last_name;
		$UserEmail		= $order->billing_email;
		$UserContact	= $order->billing_phone;

		//Date HIS
		date_default_timezone_set('Asia/Jakarta');

		$Remark		= "Transaction date " . date('Y-m-d H:i:s');
		$Lang		= "UTF-8";

		$responseUrl 	=
			WC_Compat_iPay88_online_credit::force_https(add_query_arg('wc-api', 'ipay88_online_credit_gateway', home_url('/')));

		$backendUrl 	=
			WC_Compat_iPay88_online_credit::force_https(add_query_arg('iPay88_response', 'backend', add_query_arg('wc-api', 'ipay88_online_credit_gateway', home_url('/'))));
			
		$ssignature = new ipay88_Signature_152($merchantKey . $merchantCode . $refNo . $hashAmount . $currency);
		$Signature 	= $ssignature->getSignature();

		$xfield1	= "";

		//ITEM TRANSACTION--------------------------------------------------------------------------------

		$products 	= $order->get_items();

		foreach ($products as $product) {
			//Indexing data to array > 1 dimensi
			$list['id'][] 		= "" . $product['product_id'] . "";
			$list['name'][] 	= $product['name'];
			$list['quantity'][] = "" . $product['quantity'] . "";
			$list['amount'][] 	= $product['subtotal'] . "00";

			$terms 				= get_the_terms($product['product_id'], 'product_cat');
			$list['type'][] 	= $terms[0]->name;

			//foreach ($terms as $term)
			//{
			//$list['type'][]= $term->name;
			//}

			$list['url'][] 		= get_permalink($product['product_id']);
		}
		
		//List by category in keys of array
		$item_by_id 		= $list['id'];
		$item_by_name		= $list['name'];
		$item_by_quantity	= $list['quantity'];
		$item_by_amount		= $list['amount'];
		$item_by_type		= $list['type'];
		$item_by_url		= $list['url'];

		//Make an array that save value based on key from array before
		$item_list = array_map(function () {
			return (func_get_args());
		}, $item_by_id, $item_by_name, $item_by_quantity, $item_by_amount, $item_by_type, $item_by_url);

		//Rename key of array
		foreach ($item_list as $key => $value) {
			$item_list[$key]['id'] 		= $item_list[$key]['0'];
			$item_list[$key]['name'] 		= $item_list[$key]['1'];
			$item_list[$key]['quantity'] 	= $item_list[$key]['2'];
			$item_list[$key]['amount'] 	= $item_list[$key]['3'];
			$item_list[$key]['type'] 		= $item_list[$key]['4'];
			$item_list[$key]['url'] 		= $item_list[$key]['5'];
			$item_list[$key]['ParentType'] 	= "SELLER";
			$item_list[$key]['ParentId'] 		= $merchantCode;

			unset($item_list[$key]['0']);
			unset($item_list[$key]['1']);
			unset($item_list[$key]['2']);
			unset($item_list[$key]['3']);
			unset($item_list[$key]['4']);
			unset($item_list[$key]['5']);
		}

		//Tax Fee
		$taxfee					= $order->get_total_tax();
		$taxfeeAmount			= number_format($taxfee, 2, ".", "");
		$taxfee_HashAmount		= str_replace(".", "", str_replace(",", "", $taxfeeAmount));

		//Shipping
		$shippingfee 			= $order->get_total_shipping();
		$shippingAmount			= number_format($shippingfee, 2, ".", "");
		$shipping_HashAmount	= str_replace(".", "", str_replace(",", "", $shippingAmount));

		//Voucher
		$discount 				= $order->get_total_discount();
		$discountAmount			= number_format($discount, 2, ".", "");
		$discount_HashAmount	= str_replace(".", "", str_replace(",", "", $discountAmount));

		if(! empty ($discount) && ($paymentId == "70" || $paymentId == "71"))
		{		
			WC_iPay88_online_credit::add_debug_log("Transaction Rejected , because there should be no discount for PaymentId $paymentId");
			global $woocommerce;
			wc_add_notice('Your Payment Failed. Payment using a discount, are not allowed. Please try again or use another payment option.', 'error');
			
			return wp_redirect(WC()->cart->get_checkout_url());
		}
		else 
		{
			WC_iPay88_online_credit::add_debug_log("Testing PaymentId $paymentId");
		}

		//Make into array
		$array_one 				=	array(
			"0"		=> "taxfee",
			"1"		=> "shippingfee",
			"2"		=> "discount",
		);

		$array_two				=	array(
			"0"		=> "Tax Fee",
			"1"		=> "Shipping Fee",
			"2"		=> "Discount",
		);

		$array_three			=	array(
			"0"	=> $taxfee_HashAmount,
			"1"	=> $shipping_HashAmount,
			"2"	=> "-" . $discount_HashAmount
		);

		$array_four				=	array(
			"0"	=> "1",
			"1"	=> "1",
			"2"	=> "1",
		);

		//Make an array that save value based on key from array before
		$amount_list 	= 	array_map(
										function () {
														return (func_get_args());
													}, $array_one, $array_two, $array_three, $array_four
									);

		// $amount_list = array_filter(
		// 							$array_three,
		// 							function ($val, $key) use ($array_three) {
		// 								return isset($array_three[$val]) && (
		// 									$array_three[$val] === 000
		// 								);
		// 							},
		// 							ARRAY_FILTER_USE_BOTH
		// 						);

		//Rename key of array
		foreach ($amount_list as $key => $value) {
			if(floatval($amount_list[$key]['2'])!=0){
				$amount_list[$key]['id'] 		= $amount_list[$key]['0'];
				$amount_list[$key]['name'] 	= $amount_list[$key]['1'];
				$amount_list[$key]['amount']	= $amount_list[$key]['2'];
				$amount_list[$key]['quantity'] = $amount_list[$key]['3'];
				$amount_list[$key]["ParentType"] 	= "SELLER";
				$amount_list[$key]["ParentId"] 	= $merchantCode;
			}else{
				unset($amount_list[$key]);
			}
			unset($amount_list[$key]['0']);
			unset($amount_list[$key]['1']);
			unset($amount_list[$key]['2']);
			unset($amount_list[$key]['3']);
		}

		//$firtn		=>	$order->get_shipping_first_name();
		//$shippingln	=>	$order->get_shipping_last_name();
		//$shippingc	=>	$order->get_shipping_company();
		//$shippingadd	=>	$order->get_shipping_address_1();
		//$shippingadd2	=>	$order->get_shipping_address_2();
		//$shippingcit	=>	$order->get_shipping_city();
		//$shippingst		=>	$order->get_shipping_state();
		//$shippingpc		=>	$order->get_shipping_postcode();
		//$shippingcoun	=>	$order->get_shipping_country();
		//$store_user1 = dokan()->vendor->get(get_query_var('author'));

		//print_r( $store_user1);
		//echo "======";
		//Add array
		
		//Merge into one array
		if(!empty($amount_list))
		{
		$itemTransactions = array_merge($item_list, $amount_list);
		}
		else
		{
			$itemTransactions = $item_list;
		}

		//SHIPPING AND BILLING ADDRESS--------------------------------------------------------------------

		$ShippingAddress = array(
			'FirstName'		=> $order->billing_first_name,
			'LastName'		=> $order->billing_last_name,
			'Address'		=> $order->billing_address_1,
			'City'			=> $order->billing_city,
			'State'			=> $order->billing_state,
			'PostalCode'	=> $order->billing_postcode,
			'Phone'			=> $order->billing_phone,
			'CountryCode'	=> $order->billing_country,
		);

		$BillingAddress = array(
			'FirstName'		=> $order->billing_first_name,
			'LastName'		=> $order->billing_last_name,
			'Address'		=> $order->billing_address_1,
			'City'			=> $order->billing_city,
			'State'			=> $order->billing_state,
			'PostalCode'	=> $order->billing_postcode,
			'Phone'			=> $order->billing_phone,
			'CountryCode'	=> $order->billing_country,
		);

		// The main address pieces:
		$store_address     = get_option( 'woocommerce_store_address' );
		$store_address_2   = get_option( 'woocommerce_store_address_2' );
		$store_city        = get_option( 'woocommerce_store_city' );
		$store_postcode    = get_option( 'woocommerce_store_postcode' );

		// The country/state
		$store_raw_country = get_option( 'woocommerce_default_country' );

		// Split the country/state
		$split_country = explode( ":", $store_raw_country );

		// Country and state separated:
		$store_country = $split_country[0];
		$store_state   = $split_country[1];

		// SELLER ----------------------------------------------------------------------------------------
		$Sellers = array(
			'Id'			=>	$merchantCode,
			'Name'			=>	get_bloginfo('name'),
			'Email'			=>	$this->getConfigOptionByKey('seller_email'),
			'Url'			=>  site_url(),
			'Address'		=> 
				[
				'FirstName'		=>	$this->getConfigOptionByKey('seller_first_name'),
				'LastName'		=>	$this->getConfigOptionByKey('seller_last_name'),
				'Address'		=>	$store_address,
				'City'			=>	$store_city,
				'PostalCode'	=>	$store_postcode,
				'Phone'			=>	$this->getConfigOptionByKey('seller_phone'),
				'CountryCode'	=>	'ID'
				]
		);

		//GENERATE JSON-----------------------------------------------------------------------------------

		$ipay88_Obj = new stdClass();

		$ipay88_Obj->MerchantCode 				= $merchantCode;
		$ipay88_Obj->PaymentId 					= $paymentId;
		$ipay88_Obj->Currency 					= $currency;
		$ipay88_Obj->RefNo						= $refNo;
		$ipay88_Obj->Amount						= $hashAmount;
		$ipay88_Obj->ProdDesc					= $ProdDesc;
		$ipay88_Obj->UserName					= $UserName;
		$ipay88_Obj->UserEmail					= $UserEmail;
		$ipay88_Obj->UserContact				= $UserContact;
		$ipay88_Obj->Remark						= $Remark;
		$ipay88_Obj->Lang						= $Lang;
		$ipay88_Obj->ResponseURL				= $responseUrl;
		$ipay88_Obj->BackendURL					= $backendUrl;
		$ipay88_Obj->Signature					= $Signature;
		$ipay88_Obj->xfield1					= $xfield1;
		//$ipay88_Obj->FullTransactionAmount	= $HashFullTransactionAmount;
		//$ipay88_Obj->MiscFee					= $MiscFee;

		$ipay88_Obj->itemTransactions			= $itemTransactions;
		$ipay88_Obj->ShippingAddress			= $ShippingAddress;
		$ipay88_Obj->BillingAddress				= $BillingAddress;

		//if($paymentId = "70")
		//{
			$ipay88_Obj->Sellers				= array($Sellers);
		//}


		$ipay88_JSON = json_encode($ipay88_Obj, JSON_UNESCAPED_SLASHES);

		WC_iPay88_online_credit::add_debug_log("JSON Payment Request " . print_r($ipay88_JSON,true));
		$url  		 = ($getwayUrl);
		$options 	 = array(
			'http' 	=> array(
				'header'  	=> "Content-type: application/json\r\n",
				'method'  	=> 'POST',
				"timeout" => (float) 30.0,
				'content' 	=> $ipay88_JSON
			)
		);

		$context		= stream_context_create($options);

		$result 		= file_get_contents($url, false, $context);
		
		WC_iPay88_online_credit::add_debug_log("JSON Payment Response " . print_r($result,true));

		//JSON FAILED
		if ($result === FALSE) {
			//update status payment failed + create message
			$message_failed = "iPay88 Payment Failed. Error description: Connection failed with iPay88 JSON.";

			$order->update_status('failed',  __($message_failed, 'woocommerce'));

			global $woocommerce;
			wc_add_notice('Your Payment Failed. Please try again or use another payment option.', 'error');
			return wp_redirect(WC()->cart->get_checkout_url());
		} else {
			$data = json_decode($result, true);
			//print_r ($data);
			//Status Fail
			if ($data['Status'] == "0") {
				$error_response = $data['ErrDesc'];

				//update status payment failed + create message
				$message_failed = "iPay88 Payment Failed. Json already connected. Error description: " . $error_response . ".";
				$order->update_status('failed',  __($message_failed, 'woocommerce'));

				global $woocommerce;
				wc_add_notice('Your Payment Failed. Please try again or use another payment option.', 'error');
				return wp_redirect(WC()->cart->get_checkout_url());
			}

			//Status Pending
			else {
				//Signature from iPay88
				$ipay88_Signature 		= $data['Signature'];

				//Generate Response Signature
				$MerchantCode_response 	= $merchantCode;
				$MerchantKey_response 	= $merchantKey;
				$PaymentId_response		= $data['PaymentId'];
				$Amount_response		= $data['Amount'];
				$Currency_response		= $data['Currency'];
				$Status_response		= $data['Status'];
				$merchant_signature 	= "";

				//update status payment pending + Create message
				$update_message = "iPay88 Payment Pending. Json already connected.";

				WC_Compat_iPay88_online_credit::wc_include_js('jQuery("body").block(
				{
					message: "' . __('Thank you for your order. We are now redirecting you to iPay88 to make payment.', IPAY88_ONLINE_CREDIT_TEXT_DOMAIN) . '",

					overlayCSS:
					{
						background: "#fff",
						opacity: 0.6
					},

					css:
					{
						padding:        20,
						textAlign:      "center",
						color:          "#555",
						border:         "3px solid #aaa",
						backgroundColor:"#fff",
						cursor:         "wait",
						lineHeight:		"32px",
						zIndex:         "9999999"
					}
				});

				jQuery("#submit_ipay88_payment_form").click();
				');

				return '<form method="post" action="' . $data['CheckoutURL'] . '" id="ipay88_payment_form" target="_top">
				<input type="submit" class="button-alt" id="submit_ipay88_payment_form" hidden="hidden" value="' . __('Pay via iPay88', IPAY88_ONLINE_CREDIT_TEXT_DOMAIN) . '"></form>';
				
			}
		}
	}
	
	// Process the order after payment is made
	// @param int $order_id
	// @return array
	function process_payment($order_id)
	{
		$order = wc_get_order($order_id);

		$payment_method_selected 			= $_POST['ipay88_bank_payment_method'];
		$_SESSION['posted_payment_type']	= $_POST['ipay88_bank_payment_method'];

		return array(
			'result' 	=> 'success',
			'redirect' 	=> add_query_arg('key', $order->order_key, add_query_arg(
				array(
					'order-pay' => !method_exists($order, 'get_id') ? $order->id : $order->get_id(),
					'ipay88_payment_method_selected' => $payment_method_selected
				),
				$order->get_checkout_payment_url(true)
			))
		);
	}

	// Check the response from ipay88's Servers
	function ipn_response()
	{
		global $woocommerce;
		$refNo = $_POST['RefNo'];
		$splitReffNo = explode( "-", $refNo );
		$ReffNo_Date	= $splitReffNo[0];
		$ReffNo_Number	= $splitReffNo[1];

		WC_iPay88_online_credit::add_debug_log("ipn_response " . print_r($_REQUEST,true));

		$order = wc_get_order($ReffNo_Number);

		if (!$order) {
			die('The order is Invalid!');
		}

		$merchantKey = $this->getConfigOptionByKey('merchant_key');
		$transaction = new ipay88_Transaction_152($_POST);

		$transaction->setMerchantKey($merchantKey);
		$transaction->setSignature($_POST['Signature']);

		WC_iPay88_online_credit::add_debug_log("transaction " . print_r($transaction,true));

		if ($transaction->isPaid()) {
			WC_iPay88_online_credit::add_debug_log("transaction isPaid " . print_r($transaction,true));

			$order->add_order_note(__('The Order has been Paid by ipay88 Payment Gateway - Transaction Id: ' . $_POST['TransId'], IPAY88_ONLINE_CREDIT_TEXT_DOMAIN));

			$order->payment_complete();
			$woocommerce->cart->empty_cart();

			if (isset($_REQUEST['iPay88_response'])) {
				die('RECEIVEOK');
				
			} else {
				status_header(200);

				wc_add_notice('Your Payment success.', 'success');

				wp_redirect($this->get_return_url($order));

				// Return thank you page redirect
				//return array('result'   => 'success', 'redirect' => $this->get_return_url($order));
			}
		} elseif ($transaction->isWaitingPayment()) {
			WC_iPay88_online_credit::add_debug_log("transaction isWaitingPayment " . print_r($transaction,true));

			$order->add_order_note(__('The Order is waiting payment via ipay88 Payment Gateway - Transaction Id: ' . $_POST['TransId'], IPAY88_ONLINE_CREDIT_TEXT_DOMAIN));

			$order->update_status('on-hold');

			if (isset($_REQUEST['iPay88_response'])) {
				die('PENDING');
			} else {
				wp_redirect($this->get_return_url($order));
			}
		} else {
			WC_iPay88_online_credit::add_debug_log("transaction ErrDesc " . print_r($transaction,true));

			global $woocommerce;
			wc_add_notice('Your Payment Failed. Please try again or use another payment option.', 'error');

			$order->cancel_order(__('Reason: ' . $_POST['ErrDesc'], IPAY88_ONLINE_CREDIT_TEXT_DOMAIN));

			if (isset($_REQUEST['iPay88_response'])) {
				die('NOTOK');
			} else {
				return wp_redirect(WC()->cart->get_checkout_url());
			}			
		}
	}

	// Format the two amounts we need.
	// One for hashing
	// One for request parameter
	// @param double $amount
	function format_amount($amount)
	{
		if (is_numeric($amount)) {
			$this->hash_amount 		= number_format($amount, 2, '', '');
			$this->formatted_amount = number_format($amount, 2, '', '');
		}
	}

	/**
	 * Generate the sha1 control signature. <br/>
	 * Used in both the request and the response to validate the authenticity of the message.
	 *
	 * @global object $woocommerce
	 * @param array $params The request or response parameters
	 * @param bool $is_response Are the parameters from the response message
	 * @return string The sha1 generated string
	 **/
	private function generate_sha1_signature($params, $is_response = true)
	{
		$string = '';
		if ($is_response) {
			$this->format_amount(str_replace(',', '', $params['Amount']));
			$string = $this->merchantKey . $this->merchantCode . $params['PaymentId'] . $params['RefNo'] . $this->hash_amount / 100 . $params['Currency'] . $params['Status'];
		} else {
			$string = $this->merchantKey . $this->merchantCode . $params['RefNo'] . $this->hash_amount . $params['Currency'];
		}

		//Debug log
		if ('yes' == $this->debug) WC_iPay88_online_credit::add_debug_log('Signature string is: ' . $string);

		return base64_encode($this->hex2bin(sha1($string)));
	}
}
